﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.InteropServices;   ///< To use Toradex C Win32 DLLs.

namespace vcsharp_matrixKeypad_demo
{
    public partial class Form1 : Form
    {
        [DllImport("GPIOLib_dll.dll", CharSet = CharSet.Auto)]                     ///< Importing DLL.
        public static extern bool SetPinAltFn(int pinNum, int altFn, bool dirOut); ///< External function declaration.

        [DllImport("GPIOLib_dll.dll", CharSet = CharSet.Auto)]
        public static extern void InitGPIOLib();

        [DllImport("GPIOLib_dll.dll", CharSet = CharSet.Auto)]
        public static extern bool SetPinLevel(int gpioNum, int val);

        [DllImport("GPIOLib_dll.dll", CharSet = CharSet.Auto)]
        public static extern bool GetPinLevel(int gpioNum);

        [DllImport("GPIOLib_dll.dll", CharSet = CharSet.Auto)]
        public static extern void DeInitGPIOLib();

        private const int ROW1 = 98;
        private const int ROW2 = 133;
        private const int ROW3 = 103;
        private const int ROW4 = 101;
        private const int COL1 = 97;
        private const int COL2 = 85;
        private const int COL3 = 79;
        private const int COL4 = 45;
        private const int HIGH = 1;
        private const int LOW = 0;
        private const int GPIO = -1;
        private const int totalRows = 4;
        private const int totalColumns = 4;
        private const int keyStableHitTimeout = 20;     ///< Time-out in ms to ignore the bouncing of the keystroke.
                                                        ///
        private int[] colPin = new int[4]{COL1, COL2, COL3, COL4};
        private int[] rowPin = new int[4]{ROW1, ROW2, ROW3, ROW4};

        private TextBox[] textBoxArray = new TextBox[totalColumns * totalRows];
        private bool anyBoxInGreenColor = false;

        public Form1()
        {
            InitializeComponent();
            InitGPIOLib();
            /// Sets pins dedicated for columns as output LOW.
            SetPinAltFn(COL1, GPIO, true);
            SetPinLevel(COL1, LOW);
            SetPinAltFn(COL2, GPIO, true);
            SetPinLevel(COL2, LOW);
            SetPinAltFn(COL3, GPIO, true);
            SetPinLevel(COL3, LOW);
            SetPinAltFn(COL4, GPIO, true);
            SetPinLevel(COL4, LOW);
            /// Sets pins dedicated for rows as input.
            SetPinAltFn(ROW1, GPIO, false);
            SetPinAltFn(ROW2, GPIO, false);
            SetPinAltFn(ROW3, GPIO, false);
            SetPinAltFn(ROW4, GPIO, false);
            /// Collects all text-boxes in an array.
            textBoxArray[0] = textBox1;
            textBoxArray[1] = textBox2;
            textBoxArray[2] = textBox3;
            textBoxArray[3] = textBox4;
            textBoxArray[4] = textBox5;
            textBoxArray[5] = textBox6;
            textBoxArray[6] = textBox7;
            textBoxArray[7] = textBox8;
            textBoxArray[8] = textBox9;
            textBoxArray[9] = textBox10;
            textBoxArray[10] = textBox11;
            textBoxArray[11] = textBox12;
            textBoxArray[12] = textBox13;
            textBoxArray[13] = textBox14;
            textBoxArray[14] = textBox15;
            textBoxArray[15] = textBox16;
        }

        /// Sets a particular column as High.
        private void SelectCol(int column)
        {
            int columnCount = 0;

            for (columnCount = 0; columnCount < totalColumns; columnCount++)
            {
                if (columnCount == column)
                {
                    /// Sets the column pin level HIGH.
                    SetPinLevel(colPin[columnCount], HIGH);
                }
                else
                {
                    /// Sets the column pin level LOW.
                    SetPinLevel(colPin[columnCount], LOW);
                }
            }
        }

        /// Scans for row that is High.
        private bool GetRow(ref int row)
        {
            int rowCount = 0;
            bool retVal = false;
            for (rowCount = 0; rowCount < totalRows; rowCount++)
            {
                /// Gets the level of the pin configured as row.
                retVal = GetPinLevel(rowPin[rowCount]);
                if (retVal == true)
                {
                    /// Time-out in ms to ignore the bouncing of the keystroke.
                    Thread.Sleep(keyStableHitTimeout);
                    /// Gets the level of the pin configured as row.
                    retVal = GetPinLevel(rowPin[rowCount]);
                    if (retVal == true)
                    {
                        row = rowCount;
                        break;
                    }
                }
            }
            return retVal;
        }

        /// Scans for key pressed on matrix keypad.
        private bool MatrixKeypadScan(ref int row, ref int column)
        {
            int columnCount = 0;
            bool retVal = false;
            row = 0;
            column = 0;
            while (columnCount < totalColumns)
            {
                SelectCol(columnCount);      ///< Sets column pin level High.
                retVal = GetRow(ref row);    ///< Scans for row that is High.
                if (retVal == true)
                {
                    column = columnCount;
                    break;
                }
                columnCount++;
                Thread.Sleep(5);
            }
            return retVal;
        }

        /// Shows result on UI.
        private void UpdateResult(int rowResult, int columnResult, bool keyPressed)
        {
            int count = 0;
            int resultToDisplay = 0;
            if (keyPressed == true)
            {
                /// Convert row and column number to match with text of text-boxes.
                resultToDisplay = ((rowResult + 1) * 10) + columnResult + 1;
                /// For example, if rowResult = 0 and columnResult = 0 => resultToDisplay = 11 i.e. First box selected.
            }
            for (count = 0; count < (totalColumns * totalRows); count++)
            {
                if (textBoxArray[count].Text == resultToDisplay.ToString())
                {
                    textBoxArray[count].BackColor = Color.YellowGreen;
                    anyBoxInGreenColor = true;
                }
                else
                {
                    textBoxArray[count].BackColor = Color.White;
                }
            }
        }

        private void scanButton_Click(object sender, EventArgs e)
        {
            if (scanButton.Text == "Start Scanning")
            {
                scanButton.Text = "Stop Scanning";
                updateData.Enabled = true;
            }
            else
            {
                updateData.Enabled = false;
                scanButton.Text = "Start Scanning";
                UpdateResult(0, 0, false);
            }
        }

        private void updateData_Tick(object sender, EventArgs e)
        {
            int rowOutput = 0;
            int columnOutput = 0;
            bool retVal = false;

            retVal = MatrixKeypadScan(ref rowOutput, ref columnOutput);
            if (retVal == true)
            {
                UpdateResult(rowOutput, columnOutput, true);
            }
            if (anyBoxInGreenColor == true && retVal == false)
            {
                anyBoxInGreenColor = false;
                UpdateResult(rowOutput, columnOutput, false);
            }
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            updateData.Enabled = false;
            ///< De-initializes GPIO library and frees up system resources before closing the program.
            DeInitGPIOLib();
        }
    }
}